@@ -287,14 +287,6 @@ struct controlseqs ctlseqstab[] =
          "--raw=CTLSEQS",
          "issue raw control sequence"},
 
-   { 0,  0, NULL, ANY, NULL,                    /* GET_RAW      */
-         "--get-raw=CTLSEQS",
-         "report raw control sequence"},
-
-   { 0,  0, NULL, ANY, NULL,                    /* TIMEOUT      */
-         "--timeout=DECISECONDS",
-         "timeout in deciseconds for reports"},
-
    { 0,  0, NULL, ANY, NULL,                    /* FILE_CONF    */
          "--file=FILE",
          "alternative configuration file"}
@@ -305,7 +297,6 @@ main(int argc, char** argv)
 {
    unsigned i;
    int c, ctl1, ctl2, type;
-   int timeout = 1;
    int force = 0;
    int verbose = 0;
    int configuration_missing = 0;
@@ -385,9 +376,7 @@ main(int argc, char** argv)
          {"lower"        , 0, &a, LOWER},
          {"reset"        , 0, &a, RESET},
          {"raw"          , 1, &a, RAW},
-         {"get-raw"      , 1, &a, GET_RAW},
          {"file"         , 1, &a, FILE_CONF},
-         {"timeout"      , 1, &a, TIMEOUT},
          {"force"        , 0,  0, 'f'},
          {"verbose"      , 0,  0, 'v'},
          {"version"      , 0,  0, 'u'},
@@ -442,7 +431,6 @@ main(int argc, char** argv)
                case FONT     :
                case TITLE    :
                case RAW      :
-               case GET_RAW  :
                   ctlseqstab[a].text = optarg;
                   break;
                case GET_FG       :
@@ -491,10 +479,6 @@ main(int argc, char** argv)
                   else
                      snprintf(configuration_file, sizeof(configuration_file), "%s", optarg);
                   break;
-               case TIMEOUT:
-                  timeout = atoi(optarg);
-                  timeout = timeout < 1 ? 1 : timeout;
-                  break;
                default:
                   usage(EXIT_FAILURE);
             }
@@ -528,7 +512,7 @@ main(int argc, char** argv)
       }
    }
 
-   tty_control(timeout);
+   tty_control();
 
    for (i = 0; i < NSEQ; i++) {
       type = ctlseqstab[i].type;
@@ -572,8 +556,7 @@ main(int argc, char** argv)
                      get_title(temp, sizeof(temp), verbose, ctl1);
                      break;
                   default:
-                     csi_print1(ctl1);
-                     tty_read(temp, sizeof(temp), ';');
+                     /* NOTREACHED */
                      break;
                }
                if (*temp)
@@ -592,11 +575,6 @@ main(int argc, char** argv)
 
             case ANY:
                raw_print(text);
-               if (i == GET_RAW) {
-                  tty_read(temp, sizeof(temp), '\n');
-                  if (*temp)
-                     fprintf(stdout, "%s\n", temp);
-               }
                break;
 
             default:
@@ -636,7 +614,7 @@ check_term_variable(void)
 }
 
 void
-tty_control(int vtime)
+tty_control()
 {
    ctty = NULL;
 
@@ -659,7 +637,7 @@ tty_control(int vtime)
    else
       tty_out = stdout;
 
-   set_tty_raw(vtime);
+   set_tty_raw();
 }
 
 /* find path to controlling terminal */
@@ -677,7 +655,7 @@ ctty_path(void)
 }
 
 void
-set_tty_raw(int vtime)
+set_tty_raw()
 {
    /* get and backup tty_in termios */
    tcgetattr(TTY_FILENO, &tty_ts);
@@ -687,11 +665,11 @@ set_tty_raw(int vtime)
    /* set tty raw */
    tty_ts.c_iflag = 0;
    tty_ts.c_lflag = 0;
-   tty_ts.c_cc[VMIN] = 0;
-   tty_ts.c_cc[VTIME] = vtime;
-   tty_ts.c_iflag &= ~(ICRNL | IXON);
-   tty_ts.c_lflag &= ~(ICANON | ECHO | ECHONL);
-   tcsetattr(TTY_FILENO, TCSAFLUSH, &tty_ts);
+
+   tty_ts.c_cc[VMIN] = 1;
+   tty_ts.c_cc[VTIME] = 1;
+   tty_ts.c_lflag &= ~(ICANON | ECHO);
+   tcsetattr(TTY_FILENO, TCSANOW, &tty_ts);
 }
 
 void
@@ -762,21 +740,19 @@ raw_print(char* ctlseq)
    fflush(tty_out);
 }
 
-/* read raw terminal output */
-void
-tty_read(char* output, size_t size, char esc_subst)
+/* read raw terminal output, not '\0' terminated! */
+int
+tty_read(char* output, size_t size)
 {
-   size_t i = 0;
+   int n;
 
-   /* remove ESC from tty_in */
-   read(TTY_FILENO, output, 1);
-
-   while (read(TTY_FILENO, output+i, 1) == 1) {
-      if (*(output+i) == '\033' || *(output+i) == '\007')
-         *(output+i) = esc_subst;
-      if (i++ == size-1) break;
+   n = read(TTY_FILENO, output, size-1);
+   if (n == -1) {
+      perror("read");
+      do_exit(EXIT_FAILURE);
    }
-   output[i] = '\0';
+
+   return n;
 }
 
 /* issue CSI (Control Sequence Introducer) escape sequence */
@@ -804,21 +780,27 @@ csi_print3(int ctl1, int ctl2, int ctl3)
 void
 get_title(char* title, size_t size, int verbose, int ctl1)
 {
-   static char local_temp[BUFSIZ];
-   static char local_temp2[BUFSIZ];
+   /* OSC  l title ST */
+   int n;
+   static char s[BUFSIZ];
 
    csi_print1(ctl1);
-   /* use vertical tab ('\v') as "end of title" mark */
-   tty_read(local_temp, sizeof(local_temp), '\v');
-   if (sscanf(local_temp, "%*[^l]l%[^\v]s", local_temp2) != 1) {
-      fprintf(stderr, "%s: failed to read title\n", program_name);
-      do_exit(EXIT_FAILURE);
+
+   n = tty_read(s, sizeof(s));
+   if (n == 1)                           /* get at least the OSC */
+      n += tty_read(s+1, sizeof(s)-1);
+
+   while ( !(s[n-2] == '\033' && s[n-1] == '\\') ) {
+      n += tty_read(s+n, sizeof(s)-n);
    }
-   if (!verbose)
-      snprintf(title, size, "%s", local_temp2);
+
+   s[n-2] = '\0';                       /* n-2: discard ST */
+
+   if (!verbose)                        /* s+3: discard OSC l */
+      snprintf(title, size, "%s", s+3);
    else
       snprintf(title, size, "%s=\"%s\"",
-            ctlseqstab[GET_TITLE].conf_title, local_temp2);
+            ctlseqstab[GET_TITLE].conf_title, s+3);
 }
 
 /* issue OSC (Operating System Controls) escape sequence */
@@ -844,8 +826,9 @@ osc_print(int ps1, int ps2, char* pt)
 void
 get_osc(char* osc, size_t size, int verbose, int option, int ctl1, int ctl2)
 {
-   static char local_temp[BUFSIZ];
-   static char local_temp2[BUFSIZ];
+   int n;
+   char *p;
+   static char s[BUFSIZ];
 
    char* conf_title = ctlseqstab[option].conf_title;
 
@@ -854,23 +837,24 @@ get_osc(char* osc, size_t size, int verbose, int option, int ctl1, int ctl2)
    else
       osc_print(ctl1, 0, NULL);
 
-   tty_read(local_temp, sizeof (local_temp), '\n');
-   if (ctl1 == 4) {
-      if (sscanf(local_temp, "%*[^;];%*[^;];%s", local_temp2) != 1) {
-         fprintf(stderr, "%s: failed to read %s\n", program_name, conf_title);
-         do_exit(EXIT_FAILURE);
-      }
-   } else {
-      if (sscanf(local_temp, "%*[^;];%s", local_temp2) != 1) {
-         fprintf(stderr, "%s: failed to read %s\n", program_name, conf_title);
-         do_exit(EXIT_FAILURE);
-      }
-   }
+   n = tty_read(s, sizeof(s));
+   if (n == 1)                          /* get at least the OSC */
+      n += tty_read(s+1, sizeof(s)-1);
+
+   while (s[n-1] != '\007')
+      n += tty_read(s+n, sizeof(s)-n);
+
+   s[n-1] = '\0';                       /* n-2: discard BEL */
+
+   p = strchr(s, ';');                  /* discard OSC Ps ; */
+   if (ctl1 == 4)
+      p = strchr(p+1, ';');
+   ++p;
 
    if (!verbose)
-      snprintf(osc, size, "%s", local_temp2);
+      snprintf(osc, size, "%s", p);
    else
-      snprintf(osc, size, "%s=\"%s\"", conf_title, local_temp2);
+      snprintf(osc, size, "%s=\"%s\"", conf_title, p);
 }
 
 void
@@ -902,21 +886,37 @@ set_geometry(int ctl1, int ctl2, char* geometry)
 void
 get_geometry(char* geometry, size_t size, int verbose, int ctl1, int ctl2)
 {
-   int w, h, x, y;
+   int n, w, h, x, y;
    char temp[20];
 
-   /* get height,width */
+   /* get height, width - CSI 8 ; height ; width t */
    csi_print1(ctl1);
-   tty_read(temp, sizeof(temp), ';');
-   if (sscanf(temp, "%*[^;];%d;%dt", &h, &w) != 2) {
+   n = tty_read(temp, sizeof(temp));
+   if (n == 1)                                       /* get at least the CSI */
+      n += tty_read(temp+1, sizeof(temp)-1);
+
+   while (temp[n-1] != 't')
+      n += tty_read(temp+n, sizeof(temp)-n);
+
+   temp[n-1] = '\0';                                 /* n-1: discard t */
+
+   if (sscanf(temp+4, "%d;%d", &h, &w) != 2) {       /* temp+4: discard CSI 8 ; */
       fprintf(stderr, "%s: failed to read xterm size\n", program_name);
       do_exit(EXIT_FAILURE);
    }
 
-   /* get x,y */
+   /* get x,y  - CSI 3 ; x; yt */
    csi_print1(ctl2);
-   tty_read(temp, sizeof(temp), ';');
-   if (sscanf(temp, "%*[^;];%d;%dt", &x, &y) != 2) {
+   n = tty_read(temp, sizeof(temp));
+   if (n == 1)                                       /* get at least the CSI */
+      n += tty_read(temp+1, sizeof(temp)-1);
+
+   while (temp[n-1] != 't')
+      n += tty_read(temp+n, sizeof(temp)-n);
+
+   temp[n-1] = '\0';                                 /* n-1: discard t */
+
+   if (sscanf(temp+4, "%d;%dt", &x, &y) != 2) {      /* temp+4: discard CSI 3 ; */
       fprintf(stderr, "%s: failed to read xterm position\n", program_name);
       do_exit(EXIT_FAILURE);
    }
